from typing import List

import requests
from bs4 import BeautifulSoup


def fetch_inputs_and_outputs(
        url: str,
        contest: str,
        contest_no: int) -> List[str]:
    """
    Atcoderの設問ページから入力と出力を取ってくる

    Parameters
    ----------
    url : str
        設問ページのURL
    contest : str
        コンテスト名. ABC, ARC or AGC
    contest_no : int
        コンテンストの番号

    Returns
    ----------
    inputs_and_output : List[str]
        インプットとアウトプットが交互に並んだリスト
    """
    def get_text(html):
        """
        HTMLタグ中からテキストを取り出す

        Parameters
        ----------
        html
            BeautifulSoupで定義されたResultSet

        Returns
        ----------
        lst
            取り出されたテキスト文字列のリスト
        """
        return list(map(lambda tag: tag.get_text().strip(), html))

    def split_half(lst):
        """
        リストの前半分を取り出す

        Parameters
        ----------
        lst
            半分に分けたいリスト

        Returns
        ----------
        lst_half
            引数で受け取ったリストの前半分
        """
        return lst[:int(len(lst)/2)]

    response = requests.get(url)
    soup = BeautifulSoup(response.content, 'html.parser')
    pre_list = soup.find_all('pre')
    if contest == 'ABC':
        if 1 <= contest_no <= 41:
            return get_text(pre_list[1:])
        else:
            return split_half(get_text(pre_list))[1:]
    elif contest == 'ARC':
        if 1 <= contest_no <= 57:
            return get_text(pre_list[1:])
        else:
            return split_half(get_text(pre_list))[1:]
    elif contest == 'AGC':
        return split_half(get_text(pre_list))[1:]


def translate_task(contest: str, contest_no: int, task: str) -> str:
    """
    Atcoderは第N回目コンテストのNの値によって、
    URLに含まれるタスク名がアルファベット(a, b, c, d)表される場合と
    数字(1, 2, 3, 4)で表される場合がある
    数字だった場合は、タスク名(A, B, C, D)をアルファベットの小文字に変換、
    アルファベットだった場合は小文字に変換する

    Parameters
    ----------
    contest : str
        コンテスト名. ABC, ARC or AGC
    contest_no : int
        コンテンストの番号
    task : str
        タスク名. A, B, C or D

    Returns
    ----------
    task
        タスク名(1, 2, 3, 4 または a, b, c, d)
    """
    translator = {'A': '1', 'B': '2', 'C': '3', 'D': '4'}
    if contest == 'ABC':
        if 1 <= contest_no <= 19:
            return translator[task]
        else:
            return task.lower()
    elif contest == 'ARC':
        if 1 <= contest_no <= 34:
            return translator[task]
        else:
            return task.lower()
    else:
        return task.lower()


def compose_task_url(contest: str, contest_no: int, task: str) -> str:
    """
    与えられたコンテスト名、コンテスト番号、タスク名から
    問題ページのURLを生成する

    Parameters
    ----------
    contest : str
        コンテスト名. ABC, ARC or AGC
    contest_no : int
        コンテンストの番号
    task : str
        タスク名. A, B, C or D

    Returns
    ----------
    url : str
        対応する問題のURL
    """
    host = 'https://beta.atcoder.jp/'
    return host + 'contests/' + contest.lower()\
                + '{:03d}'.format(contest_no)\
                + '/tasks/' + contest.lower()\
                + '{:03d}'.format(contest_no)\
                + '_' + task

def compose_submit_url(contest: str, contest_no: int) -> str:
    """
    与えられたコンテスト名、コンテスト番号から
    提出先のURLを生成する

    Parameters
    ----------
    contest : str
        コンテスト名. ABC, ARC or AGC
    contest_no : int
        コンテスト番号

    Returns
    ----------
    url : str
        対応する提出先のURL
    """
    host = 'https://atcoder.jp/contests/'
    return host + contest.lower() + '{:03d}'.format(contest_no) + '/submit'



import subprocess
import sys
from typing import List


def run_test(
        test_input: str,
        correct_output: str,
        command: List[str]) -> (str, bool):
    """
    テストを実行する

    Parameters
    ----------
    test_input : str
        テストのインプット
    correct_output : str
        テストの期待する正解アウトプット
    command : List[str]
        実行するコマンド

    Returns
    ----------
    user_output : str
        実行したコマンドの出力結果
    result : bool
        実行したコマンドの正誤
    """
    exec_process = subprocess.Popen(command,
                                    stdout=subprocess.PIPE,
                                    stdin=subprocess.PIPE)
    user_output = exec_process.communicate(test_input.encode())[0].decode()
    user_output = user_output.strip()
    return user_output, user_output == correct_output


def run_tests(
        test_inputs: List[str],
        correct_outputs: List[str],
        command: List[str]) -> bool:
    """
    複数のテストを実行する(実際に実行するのはrun_test)

    Parameters
    ----------
    test_inputs : List[str]
        複数のテストのインプット
    correct_outputs: List[str]
        テストの期待する複数のアウトプット
    command: List[str]
        実行するコマンド

    Returns
    ----------
    result
        複数のテストの実行結果(1つでも不正解だとFalse)
    """
    results = []
    for test_input, correct_output in zip(test_inputs, correct_outputs):
        user_output, result = run_test(test_input, correct_output, command)
        message(test_input, correct_output, user_output, result)
        results.append(result)
    return all(results)


def message(test_input, correct_output, user_output, result):
    """
    Parameters
    ----------
    test_input : str
        テストのインプット
    correct_output : str
        テストの期待する正解アウトプット
    user_output : str
        実行したコマンドの出力結果
    result : bool
        テストの実行結果
    """
    sys.stdout.write('Input       : {}\n'.format(repr(test_input)))
    sys.stdout.write('Your Answer : {}\n'.format(user_output))
    sys.stdout.write('Output      : {}\n'.format(correct_output))
    if result:
        sys.stdout.write('Congraturation!!\n')
    else:
        sys.stdout.write('Uhmmmmm...\n')
    sys.stdout.write('-' * 30 + '\n')



import unittest
from atarg import utils


class HTMLParserTest(unittest.TestCase):
    def test_fetch_inputs_and_outputs(self):
        self.assertEqual(
                utils.fetch_inputs_and_outputs(
                    'https://beta.atcoder.jp/contests/abc020/tasks/abc020_a',
                    'ABC', 20),
                ['1', 'ABC', '2', 'chokudai'])
        self.assertEqual(
                utils.fetch_inputs_and_outputs(
                    'https://beta.atcoder.jp/contests/abc001/tasks/abc001_1',
                    'ABC', 1),
                ['15\r\n10', '5', '0\r\n0', '0', '5\r\n20', '-15'])
        self.assertEqual(
                utils.fetch_inputs_and_outputs(
                    'https://beta.atcoder.jp/contests/arc057/tasks/arc057_a',
                    'ARC', 57),
                ['1000 300', '4', '6 2', '25', '567876543 0', '1999432123457'])
        self.assertEqual(
                utils.fetch_inputs_and_outputs(
                    'https://beta.atcoder.jp/contests/arc058/tasks/arc058_a',
                    'ARC', 58),
                ['1000 8\r\n1 3 4 5 6 7 8 9', '2000', '9999 1\r\n0', '9999'])
        self.assertEqual(
                utils.fetch_inputs_and_outputs(
                    'https://beta.atcoder.jp/contests/agc001/tasks/agc001_a',
                    'AGC', 1),
                ['2\r\n1 3 1 2', '3',
                    '5\r\n100 1 2 3 14 15 58 58 58 29', '135'])

    def test_translate_task(self):
        self.assertEqual(utils.translate_task('ABC', 19, 'A'), '1')
        self.assertEqual(utils.translate_task('ABC', 20, 'A'), 'a')
        self.assertEqual(utils.translate_task('ARC', 34, 'A'), '1')
        self.assertEqual(utils.translate_task('ARC', 35, 'A'), 'a')
        self.assertEqual(utils.translate_task('AGC', 1, 'A'), 'a')

    def test_compose_task_url(self):
        self.assertEqual(
                utils.compose_task_url('ABC', 20, 'a'),
                'https://beta.atcoder.jp/contests/abc020/tasks/abc020_a')

    def test_compose_submit_url(self):
        self.assertEqual(
                utils.compose_submit_url('ABC', 20),
                'https://atcoder.jp/contests/abc020/submit')


if __name__ == '__main__':
    unittest.main()



import unittest
from atarg import submit

class SubmitExecutorTest(unittest.TestCase):
    def test_extract_by_name(self):
        html_text = '''
        <html>
            <head>
                <title> Yo </title>
            </head>
            <body>
                <input type="hidden" name="test1" value="1">
                <input type="hidden" name="test2" value="2">
            </body>
        </html>
        '''
        self.assertEqual(
                submit.extract_by_name(html_text, 'test1'),
                '1')



import argparse
import sys

from atarg import utils
from atarg.testcase import run_tests
from atarg.submit import submit


def parse_arguments():
    CONTESTS = ['ABC', 'ARC', 'AGC']
    TASKS = ['A', 'B', 'C', 'D']
    parser = argparse.ArgumentParser(
            prog='atarg',
            description='Testing tool before submit for atcoder',
            epilog='end',
            add_help=True,
            )
    parser.add_argument('-s', '--submit', help='submit source code', action='store_true')
    parser.add_argument('contest', choices=CONTESTS, help='Contest name')
    parser.add_argument('contest_no', type=int, help='Contest No.')
    parser.add_argument('task', choices=TASKS, help='Task name')
    parser.add_argument(
            'command',
            help='Commands or File to solve',
            nargs='+')
    return parser.parse_args()


def main():
    args = parse_arguments()
    task = utils.translate_task(args.contest, args.contest_no, args.task)

    if args.submit:
        source_code_filename = args.command[0]
        submit(args.contest, args.contest_no, args.task, source_code_filename)
    else:
        url = utils.compose_task_url(args.contest, args.contest_no, task)
        inputs_and_outputs = utils.fetch_inputs_and_outputs(url, args.contest, args.contest_no)
        inputs = inputs_and_outputs[::2]
        outputs = inputs_and_outputs[1::2]
        run_tests(inputs, outputs, args.command)


if __name__ == '__main__':
    main()



from setuptools import setup, find_packages
from os import path

root_dir = path.abspath(path.dirname(__file__))

setup(
        name='atarg',
        version='1.1.1',
        description='Utility tools for AtCoder',
        author='Ittoh Kimura',
        author_email='kimura.itto.kd3@gmail.com',
        url='https://github.com/itto-ki/Atarg/',
        license='MIT',
        install_requires=[
            'requests',
            'beautifulsoup4'
            ],
        packages=find_packages(exclude=('tests')),
        scripts=['scripts/atarg'],
        test_suite='tests',
        classifiers=[
                    'Development Status :: 4 - Beta',
                    'Environment :: Console',
                    'License :: OSI Approved :: MIT License',
                    'Programming Language :: Python :: 3',
                    'Topic :: Internet :: WWW/HTTP',
                    'Topic :: Software Development',
                    'Topic :: Utilities',
        ],
    )



import unittest
from atarg import testcase


class ExampleExecutorTest(unittest.TestCase):
    def test_run_test(self):
        self.assertEqual(
                testcase.run_test('15\r\n10', '5', 'tests/A'), ('5', True))
        self.assertEqual(
                testcase.run_test('15\r\n14', '-15', 'tests/A'),
                ('1', False))

    def test_run_tests(self):
        self.assertEqual(testcase.run_tests(
            ['15\r\n10', '0\r\n0', '5\r\n20'],
            ['5', '0', '-15'],
            'tests/A'), True)
        self.assertEqual(testcase.run_tests(
            ['15\r\n10', '2\r\n1', '5\r\n20'],
            ['5\n', '0\n', '-15\n'],
            'tests/A'), False)



import json
import sys
import os
import webbrowser

from bs4 import BeautifulSoup
import requests

from atarg import utils
from atarg.languages import languages

ATCODER_LOGIN_URL = 'https://atcoder.jp/login'
SETTING_FILE = os.environ['HOME'] + '/.atarg.json'

def extract_by_name(html_text: str, name: str):
    """
    HTMLテキストを解析し、
    HTML要素のうち、attributeのnameがnameと一致するものの値を返す

    Parameters
    ----------
    html_text : str
        HTMLテキスト
    name : str
        name attributeの名前

    Returns
    ----------
    value
        attributeのnameがnameと一致するものの値
    """
    soup = BeautifulSoup(html_text, 'html.parser')
    try:
        value = soup.find(attrs={'name': name}).get('value')
    except AttributeError as err:
        sys.stderr.write('Attribute[{}] is not found in HTML\n'.format(name))
        sys.exit(1)
    return value


def login(username: str, password: str) -> requests.Session:
    """
    Atcoderにログインする

    Parameters
    ----------
    username : str
        ユーザ名
    password : str
        パスワード

    Returns
    ----------
    session
        ログインに成功したセッション
    """
    session = requests.Session()
    response = session.get(ATCODER_LOGIN_URL)
    csrf_token = extract_by_name(response.text, 'csrf_token')
    data = {
            'username': username,
            'password': password,
            'csrf_token': csrf_token,
            }
    response = session.post(ATCODER_LOGIN_URL, data=data)
    if 'Username or Password is incorrect.' in response.text:
        sys.stderr.write('Authentication failure\n')
        sys.exit(1)
    return session


def submit(contest: str, contest_no: int, task: str, source_code_filename: str):
    """
    Atcoderに解答を提出する

    Parameters
    ----------
    contest
        コンテスト名. ABC, ARC or AGC
    contest_no
        コンテストの番号
    task
        タスク名. A, B, C or D

    Returns
    ----------
    """
    settings = None
    with open(SETTING_FILE) as f:
        settings = json.loads(f.read())

    session = login(settings['username'], settings['password'])
    task_url = utils.compose_task_url(contest, contest_no, task)
    task_response = session.get(task_url)
    csrf_token = extract_by_name(task_response.text, 'csrf_token')
    task_screen_name = extract_by_name(task_response.text, 'data.TaskScreenName')

    with open(source_code_filename) as f:
        source_code_text = f.read()
        data = {
                'csrf_token': csrf_token,
                'data.LanguageId': languages[settings['language']],
                'sourceCode': source_code_text,
                'data.TaskScreenName': task_screen_name,
                }
        submit_url = utils.compose_submit_url(contest, contest_no)
        submit_response = session.post(submit_url, data=data)
        webbrowser.open(submit_response.url)
